==> utilary-1.0.4.tar.gz: lib/src/rectangle.dart <==
import 'dart:collection';
import 'dart:math' as math show min, max;
import 'vector.dart';

class Rectangle extends IterableBase<Vector> {
  ///Return a rectangle that is the intersecton of [a] and [b].
  ///Might return `null` if [a] and [b] don't intersect.
  static Rectangle intersection(Rectangle a, Rectangle b) {
    int left = math.max(a.x, b.x);
    int right = math.min(a.x, b.x);
    int top = math.max(a.y, b.y);
    int bottom = math.min(a.y, b.y);

    int width = math.max(0, right - left);
    int height = math.max(0, bottom - top);

    if (width == 0 && height == 0) return null;

    return Rectangle(left, top, width, height);
  }

  final Vector pos;
  final Vector size;

  const Rectangle.fromPosAndSize(this.pos, this.size);

  Rectangle(int x, int y, int width, int height)
      : pos = Vector(x, y),
        size = Vector(width, height);

  Rectangle.fromTopLeftBottomRight(int top, int left, int bottom, int right)
      : pos = Vector(left, top),
        size = Vector(right - left, bottom - top);

  int get x => pos.x;
  int get y => pos.y;
  int get width => size.x;
  int get height => size.y;

  int get left => math.min(x, x + width);
  int get right => math.max(x + width, x);
  int get top => math.min(y, y + height);
  int get bottom => math.max(y + height, y);

  Vector get center => Vector((left + right) ~/ 2, (left + right) ~/ 2);

  int get area => size.area;

  ///Iterate through every point of this rectangle
  _RectangleIterator get iterator => _RectangleIterator(this);

  Rectangle inflate(int amount) => Rectangle(x - amount, y - amount, width + 2 * amount, height + 2 * amount);

  bool operator ==(Object o) {
    if (o is Rectangle) return o.pos == pos && o.size == size;

    return false;
  }

  bool contains(Object o) {
    if (o is Vector) return _containsVector(o);
    if (o is Rectangle) return _containsRectangle(o);

    throw ArgumentError("Argument must be of type 'Vector' or 'Rectangle'");
  }

  bool _containsRectangle(Rectangle o) => o.x >= left && o.x < right && o.y >= top && o.y < bottom;

  bool _containsVector(Vector o) => o.x >= left && o.x < right && o.y >= top && o.y < bottom;

  ///Returns the minimal distance between this rectangle
  ///and [other]. Returns -1 if they overlap.
  int distanceTo(Rectangle other) {
    if (intersects(other)) return -1;

    int vertical = -1;
    if (top >= other.bottom)
      vertical = top - other.bottom;
    else if (bottom < other.top) vertical = other.top - bottom;

    int horizontal = -1;
    if (left > other.right)
      horizontal = left - other.right;
    else if (right < other.left) horizontal = other.left - right;

    if (vertical != -1 && horizontal != -1) return vertical + horizontal;
    if (vertical == -1) return horizontal;
    if (horizontal == -1) return vertical;

    return vertical + horizontal;
  }

  Iterable<Vector> perimeter() {
    List<Vector> points = <Vector>[];

    //points on the top side
    for (int x = left; x < right; x++) points.add(Vector(x, top));

    //points on the right side
    for (int y = top + 1; y < bottom; y++) points.add(Vector(right - 1, y));

    //points on the bottom side
    for (int x = right - 2; x >= left; x--) points.add(Vector(x, bottom - 1));

    //points on the left side
    for (int y = bottom - 2; y > top; y--) points.add(Vector(left, y));

    return points;
  }

  ///Returns whether [o] intersects with this rectangle
  bool intersects(Rectangle o) {
    if (left > o.right) return false;
    if (right < o.left) return false;
    if (top > o.bottom) return false;
    if (bottom < o.top) return false;

    return true;
  }
}

class _RectangleIterator implements Iterator<Vector> {
  final Rectangle _rect;
  int _x;
  int _y;

  _RectangleIterator(this._rect)
      : _x = _rect.left - 1,
        _y = _rect.top;

  Vector get current => Vector(_x, _y);

  bool moveNext() {
    _x++;

    if (_x >= _rect.right) {
      _x = _rect.left;
      _y++;
    }

    return _y < _rect.bottom;
  }
}

==> utilary-1.0.7.tar.gz: lib/src/rectangle.dart <==
import 'dart:collection';
import 'dart:math' as math show min, max;
import 'vector.dart';

class Rectangle extends IterableBase<Vector> {
  ///Return a rectangle that is the intersecton of [a] and [b].
  ///Might return `null` if [a] and [b] don't intersect.
  static Rectangle intersection(Rectangle a, Rectangle b) {
    int left = math.max(a.x, b.x);
    int right = math.min(a.x, b.x);
    int top = math.max(a.y, b.y);
    int bottom = math.min(a.y, b.y);

    int width = math.max(0, right - left);
    int height = math.max(0, bottom - top);

    if (width == 0 && height == 0) return null;

    return Rectangle(left, top, width, height);
  }

  final Vector pos;
  final Vector size;

  const Rectangle.fromPosAndSize(this.pos, this.size);

  Rectangle(int x, int y, int width, int height)
      : pos = Vector(x, y),
        size = Vector(width, height);

  Rectangle.fromTopLeftBottomRight(int top, int left, int bottom, int right)
      : pos = Vector(left, top),
        size = Vector(right - left, bottom - top);

  int get x => pos.x;
  int get y => pos.y;
  int get width => size.x;
  int get height => size.y;

  int get left => math.min(x, x + width);
  int get right => math.max(x + width, x);
  int get top => math.min(y, y + height);
  int get bottom => math.max(y + height, y);

  Vector get center => Vector((left + right) ~/ 2, (left + right) ~/ 2);

  int get area => size.area;

  ///Iterate through every point of this rectangle
  _RectangleIterator get iterator => _RectangleIterator(this);

  Rectangle inflate(int amount) => Rectangle(
      x - amount, y - amount, width + 2 * amount, height + 2 * amount);

  bool operator ==(Object o) {
    if (o is Rectangle) return o.pos == pos && o.size == size;

    return false;
  }

  bool contains(Object o) {
    if (o is Vector) return _containsVector(o);
    if (o is Rectangle) return _containsRectangle(o);

    throw ArgumentError("Argument must be of type 'Vector' or 'Rectangle'");
  }

  bool _containsRectangle(Rectangle o) =>
      o.x >= left && o.x < right && o.y >= top && o.y < bottom;

  bool _containsVector(Vector o) =>
      o.x >= left && o.x < right && o.y >= top && o.y < bottom;

  ///Returns the minimal distance between this rectangle
  ///and [other]. Returns -1 if they overlap.
  int distanceTo(Rectangle other) {
    if (intersects(other)) return -1;

    int vertical = -1;
    if (top >= other.bottom)
      vertical = top - other.bottom;
    else if (bottom < other.top) vertical = other.top - bottom;

    int horizontal = -1;
    if (left > other.right)
      horizontal = left - other.right;
    else if (right < other.left) horizontal = other.left - right;

    if (vertical != -1 && horizontal != -1) return vertical + horizontal;
    if (vertical == -1) return horizontal;
    if (horizontal == -1) return vertical;

    return vertical + horizontal;
  }

  Iterable<Vector> perimeter() {
    List<Vector> points = <Vector>[];

    //points on the top side
    for (int x = left; x < right; x++) points.add(Vector(x, top));

    //points on the right side
    for (int y = top + 1; y < bottom; y++) points.add(Vector(right - 1, y));

    //points on the bottom side
    for (int x = right - 2; x >= left; x--) points.add(Vector(x, bottom - 1));

    //points on the left side
    for (int y = bottom - 2; y > top; y--) points.add(Vector(left, y));

    return points;
  }

  ///Returns whether [o] intersects with this rectangle
  bool intersects(Rectangle o) {
    if (left > o.right) return false;
    if (right < o.left) return false;
    if (top > o.bottom) return false;
    if (bottom < o.top) return false;

    return true;
  }
}

class _RectangleIterator implements Iterator<Vector> {
  final Rectangle _rect;
  int _x;
  int _y;

  _RectangleIterator(this._rect)
      : _x = _rect.left - 1,
        _y = _rect.top;

  Vector get current => Vector(_x, _y);

  bool moveNext() {
    _x++;

    if (_x >= _rect.right) {
      _x = _rect.left;
      _y++;
    }

    return _y < _rect.bottom;
  }
}
